@@ -1,6 +1,9 @@
 <?php
 define( "WP_INSTALLING", true );
 require( 'wp-config.php' );
+
+add_action( 'wp_head', 'signuppageheaders' ) ;
+
 require( 'wp-blog-header.php' );
 require_once( ABSPATH . WPINC . '/registration.php' );
 
@@ -11,6 +14,10 @@ if( is_array( get_site_option( 'illegal_names' )) && $_GET[ 'new' ] != '' && in_
 
 do_action("signup_header");
 
+function signuppageheaders() {
+	echo "<meta name='robots' content='noindex,nofollow'>\n";
+}
+
 if( $current_blog->domain . $current_blog->path != $current_site->domain . $current_site->path ) {
 	header( "Location: http://" . $current_site->domain . $current_site->path . "wp-signup.php" );
 	die();
@@ -85,7 +92,8 @@ function show_blog_form($blog_id = '', $blog_title = '', $errors = '') {
 ?>
 <tr>
 <th scope="row"  valign="top"><?php _e('Privacy:') ?></th>
-<td><label><input type="checkbox" name="blog_public" value="1" checked="checked" /> <?php _e('I would like my blog to appear in search engines like Google and Technorati, and in public listings around this site.'); ?></label></td>
+<td><?php _e('I would like my blog to appear in search engines like Google and Technorati, and in public listings around this site.'); ?> 
+<label><input type="radio" name="blog_public" value="1" <?php if( !isset( $_POST[ 'blog_public' ] ) || $_POST[ 'blog_public' ] == '1' ) { ?>checked="checked"<?php } ?> /> <strong>Yes</strong> </label> <label><input type="radio" name="blog_public" value="0" <?php if( isset( $_POST[ 'blog_public' ] ) && $_POST[ 'blog_public' ] == '0' ) { ?>checked="checked"<?php } ?> /><strong>No</strong> </label> <br />
 </tr>
 <?php
 do_action('signup_blogform', $errors);
@@ -194,6 +202,10 @@ function signup_another_blog($blog_id = '', $blog_title = '', $errors = '') {
 
 function validate_another_blog_signup() {
 	global $current_user, $blog_id, $blog_title, $errors, $domain, $path;
+	$current_user = wp_get_current_user();
+	if( !is_user_logged_in() ) {
+		die();
+	}
 
 	$result = validate_blog_form();
 	extract($result);
@@ -212,7 +224,7 @@ function validate_another_blog_signup() {
 
 function confirm_another_blog_signup($domain, $path, $blog_title, $user_name, $user_email, $meta) {
 ?>
-<h2><?php printf(__('%s Is Yours'), $domain.$path ) ?></h2>
+<h2><?php printf(__('The blog %s is yours.'), $domain.$path ) ?></h2>
 <p><?php printf(__('<a href="http://%1$s">http://%2$s</a> is your new blog.  <a href="%3$s">Login</a> as "%4$s" using your existing password.'), $domain.$path, $domain.$path, "http://" . $domain.$path . "wp-login.php", $user_name) ?></p>
 <?php
 	do_action('signup_finished');
@@ -291,7 +303,7 @@ function validate_user_signup() {
 
 function confirm_user_signup($user_name, $user_email) {
 ?>
-<h2><?php printf(__('%s Is Your New Username'), $user_name) ?></h2>
+<h2><?php printf(__('%s is your new username'), $user_name) ?></h2>
 <p><?php _e('But, before you can start using your new username, <strong>you must activate it</strong>.') ?></p>
 <p><?php printf(__('Check your inbox at <strong>%1$s</strong> and click the link given.  '),  $user_email) ?></p>
 <p><?php _e('If you do not activate your username within two days, you will have to sign up again.'); ?></p>
@@ -358,7 +370,7 @@ function validate_blog_signup() {
 
 function confirm_blog_signup($domain, $path, $blog_title, $user_name, $user_email, $meta) {
 ?>
-<h2><?php printf(__('%s Is Yours'), $domain.$path) ?></h2>
+<h2><?php printf(__('The blog %s is yours'), $domain.$path) ?></h2>
 <p><?php _e('But, before you can start using your blog, <strong>you must activate it</strong>.') ?></p>
 <p><?php printf(__('Check your inbox at <strong>%s</strong> and click the link given.  '),  $user_email) ?></p>
 <p><?php _e('If you do not activate your blog within two days, you will have to sign up again.'); ?></p>
@@ -367,12 +379,17 @@ function confirm_blog_signup($domain, $path, $blog_title, $user_name, $user_emai
 }
 
 // Main
-$active_signup = 'all';
+$active_signup = get_site_option( 'registration' );
+if( !$active_signup )
+	$active_signup = 'all';
+
 $active_signup = apply_filters( 'wpmu_active_signup', $active_signup ); // return "all", "none", "blog" or "user"
 
+if( is_site_admin() ) {
+	echo "<div style='background: #faf; font-weight: bold; border: 1px solid #333; margin: 2px; padding: 2px'>Greetings Site Administrator! You are currently allowing '$active_signup' registrations. To change or disable registration go to your <a href='wp-admin/wpmu-options.php'>Options page</a>.</div>";
+}
+
 $newblogname = isset($_GET['new']) ? strtolower(preg_replace('/^-|-$|[^-a-zA-Z0-9]/', '', $_GET['new'])) : null;
-if( $_POST['blog_public'] != 1 )
-	$_POST['blog_public'] = 0;
 
 if( $active_signup == "none" ) {
 	_e( "Registration has been disabled." );
@@ -400,6 +417,8 @@ switch ($_POST['stage']) {
 			signup_another_blog($newblogname);
 		} elseif( is_user_logged_in() == false && ( $active_signup == 'all' || $active_signup == 'user' ) ) {
 			signup_user( $newblogname, $user_email );
+		} elseif( is_user_logged_in() == false && ( $active_signup == 'blog' ) ) {
+			_e( "I'm sorry. We're not accepting new registrations at this time." );
 		} else {
 			_e( "You're logged in already. No need to register again!" );
 		}
